/*
 *  Copyright (C) 2009
 *      Authors (alphabetical) :
 *              mrmoku (Klaus Kurzmann, mok@fluxnetz.de)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU Public License as published by
 *  the Free Software Foundation; version 2 of the license.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser Public License for more details.
 */


#include <dbus/dbus-glib.h>
#include <dbus/dbus-glib-bindings.h>
#include "frameworkd-glib-opimd-calls.h"
#include "frameworkd-glib-opimd-dbus.h"
#include "dbus/calls.h"

DBusGProxy *opimdCallsBus = NULL;

typedef struct {
	void (*callback) (GError *, const int, gpointer);
	gpointer userdata;
} opimd_calls_get_new_missed_calls_t;

void
opimd_calls_get_new_missed_calls_callback(DBusGProxy *proxy, const int amount,
		GError *dbus_error, gpointer userdata)
{
	opimd_calls_get_new_missed_calls_t *data =
		(opimd_calls_get_new_missed_calls_t *)userdata;
	GError *error = NULL;

	if (data->callback != NULL) {
		if (dbus_error != NULL)
			error = dbus_handle_errors(dbus_error);
		data->callback(error, amount, data->userdata);
		if (error != NULL)
			g_error_free(error);
	}
	if (dbus_error != NULL)
		g_error_free(dbus_error);

	g_free(data);
}


void
opimd_calls_get_new_missed_calls(
		void (*callback)(GError *, const int, gpointer),
		gpointer userdata)
{
	dbus_connect_to_opimd_calls();
	opimd_calls_get_new_missed_calls_t *data =
		g_malloc(sizeof(opimd_calls_get_new_missed_calls_t));
	data->callback = callback;
	data->userdata = userdata;
	org_freesmartphone_PIM_Calls_get_new_missed_calls_async(opimdCallsBus,
			opimd_calls_get_new_missed_calls_callback,
			data);
}


void
opimd_calls_new_missed_calls_handler(DBusGProxy *proxy,
		const int amount,
		gpointer userdata)
{
	void (*callback) (const int) = NULL;

	callback = userdata;

	if (callback != NULL)
		(*callback)(amount);
}

